/**
 * Title:       Abstract Durable JMS Topic subscriber for ASETS project (ASE Trading System)
 * ClassName:   AbstractDurableTopicSubscriber.java
 * Created:     January 22, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract durable Topic subscriber within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractDurableTopicSubscriber implements javax.jms.MessageListener
{
    private TopicConnection connection = null;
    private String userID = null;

    // Constructor used to initialize AbstractDurableTopicSubscriber
    public AbstractDurableTopicSubscriber(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.TopicConnectionFactory topicConnectionFactory = new com.sun.messaging.TopicConnectionFactory();
        topicConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        TopicConnection topicConnection = topicConnectionFactory.createTopicConnection();
        topicConnection.setClientID(userID);

        // Create JMS session object
        TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create a JMS topic
        Topic subTopic = topicSession.createTopic(topicName);

        // Create a JMS durable subscriber
        TopicSubscriber topicDurableSubscriber = topicSession.createDurableSubscriber(subTopic, userID, null, true);

        // Set a JMS message listener
        topicDurableSubscriber.setMessageListener(this);

        // Intialize the AbstractTopicSubscriber application variables
        this.connection = topicConnection;
        this.userID = userID;

        // Start the JMS connection; allows messages to be captured
        connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
        if (connection != null)
            connection.close();
    }

    public String getUserID()
    {
        return this.userID;
    }
}

