/**
 * Title:       Control queue receiver with User reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   OrderListReceiverWithReply.java
 * Created:     November 25, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class OrderListReceiverWithReply extends AbstractQueueReceiver
{
    public OrderListReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getObjectProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        (message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.CLIENT_ORDER_LIST_REQUEST.toString()) ||
                         message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.MARKET_ORDER_LIST_REQUEST.toString()) ||
                         message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.INSTRUMENT_ORDER_LIST_REQUEST.toString())
                         ))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply OrderList reply
                    OrderList replyOrderList = processOrderListRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (OrderList)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyOrderList);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER_LIST.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send OrderList
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.send(replyMessage);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected OrderList processOrderListRequest(Control control) throws Exception
    {
        // Process Control request message
        // for either
        //      control.setControlType(Enums.ControlType.CLIENT_ORDER_LIST_REQUEST);
        // or
        //      control.setControlType(Enums.ControlType.MARKET_ORDER_LIST_REQUEST);

        OrderList replyOrderList = new OrderList();

        // TO DO in the the derived class

        return replyOrderList;
    }
}
