/**
 * Title:       OrderList request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   OrderListSenderWithReply.java
 * Created:     November 25, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class OrderListSenderWithReply extends AbstractQueueSenderWithReply
{
    public OrderListSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for ControlList message reply
    public OrderList sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message
        // with either
        //      control.setControlType(Enums.ControlType.CLIENT_ORDER_LIST_REQUEST);
        // or
        //      control.setControlType(Enums.ControlType.MARKET_ORDER_LIST_REQUEST);

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", control.getControlType().toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process OrderList reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER_LIST.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    OrderList replyOrderList = (OrderList)replyObject.getObject();

                    return replyOrderList;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }
        
        return null;
    }
}

