/**
 * Title:       Order queue sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   OrderSenderWithReply.java
 * Created:     November 24, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class OrderSenderWithReply extends AbstractQueueSenderWithReply
{
    public OrderSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send an Order message, waiting for reply
    public Order sendOrderGetReply(Order order) throws JMSException, Exception
    {
        // Construct Order request message
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(order);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER.toString());
        message.setStringProperty("USER_ID", order.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Order sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process Order reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(order.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    Order replyOrder = (Order)replyObject.getObject();

                    return replyOrder;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        return null;
    }
}

