/**
 * Title:       Control queue receiver with User reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   CARDRectangleListReceiverWithReply.java
 * Created:     March 10, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api.collateral;

import javax.jms.*;
import asets.api.*;


public class CARDRectangleListReceiverWithReply extends AbstractQueueReceiver
{
    public CARDRectangleListReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.CARD_LIST_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply CARDRectangleList reply
                    CARDRectangleList replyCARDRectangleList = processCARDRectangleListRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (CARDRectangleList)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyCARDRectangleList);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.CARD_LIST.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send CARDRectangleList
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.send(replyMessage);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected CARDRectangleList processCARDRectangleListRequest(Control control) throws Exception
    {
        CARDRectangleList replyCARDRectangleList = new CARDRectangleList();

        // TO DO in the the derived class

        return replyCARDRectangleList;
    }
}
