/**
 * Title:       CARDRectangleList request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   CARDRectangleListSenderWithReply.java
 * Created:     March 10, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api.collateral;

import javax.jms.*;
import asets.api.*;


public class CARDRectangleListSenderWithReply extends AbstractQueueSenderWithReply
{
    public CARDRectangleListSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for ControlList message reply
    public CARDRectangleList sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message
        // control.setControlType(Enums.ControlType.CARD_LIST_REQUEST);

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.CARD_LIST_REQUEST.toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.send(message);

        // Wait and process CARDRectangleList reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(30000);
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CARD_LIST.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    CARDRectangleList replyCARDRectangleList = (CARDRectangleList)replyObject.getObject();

                    return replyCARDRectangleList;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        return null;
    }
}

